--------------------------------------------------------------------------------
-- Company: Trenz Electronic GmbH
-- Engineer: John Hartfiel
--
-- TE0701 CPLD U14 
--
--
-- Revision     : 05
-- Release Date : 2016.11.22
-- Supported PCB Revision: 03,04,05,06
--
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library machxo2;
use machxo2.all;


ENTITY top IS
   PORT (
    -- Detect Revision 06 or newer (bank must be powered)
    REVISION_DETECION     : in std_logic;  
    -- detect power on IO  with pullup on unused io
    VCCIO3_PWR_DETECION   : in std_logic;  
      -- 
		-- JTAG in from FTDI
		--
		M_TMS         		: IN std_logic;
		M_TCK         		: IN std_logic;
		M_TDI         		: IN std_logic;
		M_TDO         		: OUT std_logic;
	    --
		-- JTAG out to Module
		--
		C_TMS         		: OUT std_logic;
		C_TCK         		: OUT std_logic;
		C_TDI         		: OUT std_logic;
		C_TDO         		: IN std_logic;
		-- JTAG to FMC
		FMC_TMS        		: OUT std_logic;
		FMC_TCK        		: OUT std_logic;
		FMC_TDI        		: OUT std_logic;
		FMC_TDO        		: IN std_logic;

		-- Pushbuttons
		S1                  : IN std_logic;
		S2                  : IN std_logic; --global reset
		
		-- TE0701-03+ only
		-- Mode switches
		CM0                 : IN std_logic;
		CM1                 : IN std_logic;
    -- only REV06 and newer --> VADJ selection to S4
		CM2                 : IN std_logic;
    
		
		-- TE70xx control pins, 3.3V IN power domain
		NOSEQ               : INOUT std_logic;
		EN1                 : OUT std_logic;
		RESIN               : OUT std_logic;
		
		MODE                : OUT std_logic; 
		PGOOD               : INOUT std_logic;
		
		-- HDMI	
		HDMI_SDA                 : INOUT std_logic;
		HDMI_SCL                 : INOUT std_logic;
	
		
		
		-- 3.3V out, or PGOOD domain, should be driven low not powered?
		MIO10               : INOUT std_logic; --	 	
		MIO11               : INOUT std_logic; --
		
		MIO12               : INOUT std_logic; --
		MIO13               : INOUT std_logic; --
		
		MIO14               : INOUT std_logic; -- UART0.RX << BDBUS0
		MIO15               : IN std_logic;    -- UART0.TX >> BDBUS1
		
		-- Module interface signals
		-- 3 Wire I2C to FMC I2C
		--
		Y0                 : IN std_logic;   -- i2c scl
		Y1                 : OUT std_logic;  -- i2c sda out
		Y5                 : IN std_logic;   --  i2c sda in
		-----------------------------------------------------
		
		--
		-- RGPIO Signals, CPLD is Clock Slave!
		--
		Y2                 : IN std_logic;   -- 
		Y3                 : OUT std_logic;  -- 
		Y4                 : IN std_logic;   -- 
		
		
		------------------------------------------------------
		Y6                 : IN std_logic;   -- LED1/RFU
		X6                 : IN std_logic;   -- LED2 :( can not use if FMC disabled!
		
	
		--
		--
		--		
		ADBUS4              : OUT std_logic; 
		ADBUS7              : IN std_logic; 

		ACBUS4              : IN std_logic; 
		ACBUS5              : IN std_logic; 
		
		BDBUS0              : IN std_logic;  -- TX from FTDI
		BDBUS1              : OUT std_logic; -- RX to FTDI
		

		-- misc control and status
		USB_OC              : IN std_logic;
		SD_DETECT           : IN std_logic;
		SD_WP               : IN std_logic;
		SEL_SD              : OUT std_logic;
		
		VID0                : INOUT std_logic;
		VID1                : INOUT std_logic;
		VID2                : INOUT std_logic;
		
		
		
		FMC_SDA             : INOUT std_logic;
		FMC_SCL             : OUT std_logic;

		EN_FMC              : OUT std_logic; -- Enable 3.3V FMC and adjustable DCDC
		PG_C2M              : OUT std_logic; -- Signal Power good to FMC
		FMC_PRSNT           : IN std_logic;  -- 0 if FMC card is present
		POK_FMC             : IN std_logic;  -- Power good signal from FMC VADJ DCDC

		-- LED's in MAGJACK
		PHY_LED1            : OUT std_logic;
		PHY_LED2            : OUT std_logic;

		-- extra pins in MIO bank1 PMod J2
		PX6                 : INOUT std_logic;
		PX7                 : INOUT std_logic;

	    -- 3.3V IN powered
		LED1                : OUT std_logic;
		LED2                : OUT std_logic;
		LED3                : OUT std_logic;
		LED4                : OUT std_logic;
		
		-- FMC VADJ powered !!
		LED5                : OUT std_logic;
		LED6                : OUT std_logic;
		LED7                : OUT std_logic;
		LED8                : OUT std_logic;

       -- Reset_n				: in std_logic;

		dummy               : OUT std_logic --unused io
      );
END ENTITY top;
--
ARCHITECTURE translated OF top IS

component ddsrpi_slave is
    Port ( RX_CLK : in STD_LOGIC;
           RX_DATA : in STD_LOGIC;
           TX_DATA : out STD_LOGIC;
           user_clk : out STD_LOGIC;
           gpio_out : out STD_LOGIC_VECTOR (31 downto 0);
           gpio_in : in STD_LOGIC_VECTOR (31 downto 0);
           resetn : in STD_LOGIC;
           ready : out STD_LOGIC);
end component ddsrpi_slave;


COMPONENT OSCH
	-- synthesis translate_off
	GENERIC (NOM_FREQ: string := "24.18");
	-- synthesis translate_on
	PORT ( STDBY :IN std_logic;
	OSC :OUT std_logic;
	SEDSTDBY :OUT std_logic);
END COMPONENT OSCH;

 attribute NOM_FREQ : string;
 attribute NOM_FREQ of MyInternalOSC : label is "24.18";


	COMPONENT PORESET
	PORT(
		SYSRST : IN std_logic;
		CLK : IN std_logic;          
		RST : OUT std_logic
		);
	END COMPONENT;


component I2C_to_GPIO8
    port (
		sda: inout  std_logic; 
        sclk: in  std_logic;
		
        GPIO_input: in  std_logic_vector(7 downto 0);
        GPIO_output: out  std_logic_vector(7 downto 0)
		);
end component;  
  
signal POR_DONE: std_logic;
signal por_cnt: std_logic_vector(31 downto 0);

signal POR_DELAYED: std_logic;

-- signal USB_JTAG_RST: std_logic;

signal clk: std_logic;
signal cnt: std_logic_vector(31 downto 0);



signal osc_clk: std_logic;

-- power ON, drive all output low if no powered!
signal PON: std_logic;
signal POWER_FAIL: std_logic;

signal BOOTMODE: std_logic;

signal MODE0: std_logic;
signal MODE1: std_logic;
signal MODE2: std_logic;
signal MODE3: std_logic;

signal ext_reset_n  : std_logic; 		

signal rgpio_clk : std_logic; 		
signal rgpio_txd : std_logic; 		
signal rgpio_rxd : std_logic; 		

signal rgpio_out : std_logic_vector(31 downto 0); 		
signal rgpio_in : std_logic_vector(31 downto 0); 		

signal rgpio_active : std_logic; 		
signal rgpio_active_code : std_logic_vector(3 downto 0) := "1010"; 	

signal GPIO_input: std_logic_vector(7 downto 0);
signal GPIO_output: std_logic_vector(7 downto 0);

signal unlimit_cnt: std_logic_vector(31 downto 0);
signal vadj_led : std_logic;

signal rev06plus_led : std_logic;


signal vid_det : std_logic;
signal c_vid_det : std_logic;
signal n_vid_det : std_logic;

BEGIN
  ----------------------------------------------------------
  -- dummi pin to prevent removing from lattice software
	dummy 		<= FMC_TDO or ACBUS4 or ACBUS5 or ADBUS7 or CM2 or rev06plus_led; --
  ----------------------------------------------------------
  --revision detection:
	process (POR_DONE, Clk)
	begin
		if POR_DONE = '0' then
			unlimit_cnt <= X"00000000";
		elsif Clk'event and Clk = '1' then
			unlimit_cnt <= unlimit_cnt + X"00000001";
		end if;
	end process;
  ----------------------------------------------------------
  --revision detection, currently not used
  --REV06+ when VADJ ON and REVISION_DETECION zero
  --REV05- when VADJ ON and REVISION_DETECION on
  --Blink, not detectable, bank power is disabled
  rev06plus_led <= '1' when VCCIO3_PWR_DETECION='1' and REVISION_DETECION='0' else '0' when VCCIO3_PWR_DETECION='1' and REVISION_DETECION='1' else unlimit_cnt(23);
  ----------------------------------------------------------
  --HDMI I2C
  
  My_GPIO: I2C_to_GPIO8 port map (
    sda 	=> HDMI_SDA,
    sclk 	=> HDMI_SCL,
  
    GPIO_input =>  GPIO_input(7 downto 0),
    GPIO_output => GPIO_output(7 downto 0)
  );
  GPIO_input(7 downto 3) <= GPIO_output(7 downto 3);
	GPIO_input(0) <= c_vid_det;
	GPIO_input(1) <= MODE3;
  GPIO_input(2) <= POK_FMC;
  
  
  ----------------------------------------------------------
	-- FMC I2C mux, just remap "3 wire split i2c" to to normal I2C
	--
	FMC_SDA   	<= '0' when Y5='0' else 'Z';
	FMC_SCL 	  <= Y0 and PON;
	Y1 			    <= FMC_SDA;

  ----------------------------------------------------------
  --CM0, CM1 Modi
	MODE0 <= '1' when CM1='1' and CM0='1' else '0'; -- Fixed 1.8V                 on REV3T5 Mode
	MODE1 <= '1' when CM1='0' and CM0='1' else '0'; -- Fixed 2.5V                 on REV3T5 Mode
	MODE2 <= '1' when CM1='1' and CM0='0' else '0'; -- Fixed 3.3V                 on REV3T5 Mode
	MODE3 <= '1' when CM1='0' and CM0='0' else '0'; -- Use I2C to select voltage  on REV3T5 Mode

  ----------------------------------------------------------
	-- When SD card in Slot force BOOTMODE to SD!
	BOOTMODE <= SD_DETECT;
	MODE 	 <= BOOTMODE;
	
	-- Select MIO mux to SD or Pmod
	-- Select SD card (not pmod J2)
	-- note: SD card in pmod J2 can be used as non bootable SD or eMMC card (not bootable)
	--
  ----------------------------------------------------------
  --SD MUX Module or PMOD, 
	SEL_SD   <= '0'; --Fix Module;	
  ----------------------------------------------------------
  --UART Connection
	MIO14 <= 'Z' when PON = '0' else BDBUS0;
	BDBUS1 <= MIO15;

  ----------------------------------------------------------
  --  Reset
	ext_reset_n <= S2;
	RESIN 	<= ext_reset_n; 
  ----------------------------------------------------------
  --  Power
	--PON 	<= S2 and POR_DELAYED;
	PON 	<= POR_DELAYED;
	
	-- Keep powered!
	EN1 	<= '1'; --POR_DELAYED;
	-- Power/Reset Control
  NOSEQ <= 'Z';

	-- FMC Related signals
	PG_C2M	 	<= GPIO_output(3) and PON when MODE3='1' and c_vid_det='1' else '1'; -- I2C in MODE3 or force on otherwise
	EN_FMC 		<= GPIO_output(7) and PON when MODE3='1' and c_vid_det='1' else '1'; -- I2C in MODE3 or force on otherwise
	
  --
	-- 000 = 3.3V 
	-- 001 = 2.5V
	-- 010 = 1.8V
	-- 011 = 1.5V
	--
	-- use dip switch M2, M1
	-- 00 1.8V
	-- 01 2.5V
	-- 10 3.3V
	-- 11 --> I2C controlled
	--
	VID0  <= 'Z' when c_vid_det='0' or PON='0' else GPIO_output(4) when MODE3='1' else '1' when MODE1='1' else '0';
	VID1  <= 'Z' when c_vid_det='0' or PON='0' else GPIO_output(5) when MODE3='1' else '1' when MODE0='1' else '0';
	VID2  <= 'Z' when c_vid_det='0' or PON='0' else GPIO_output(6) when MODE3='1' else '0';
	
  vadj_led <= unlimit_cnt(23) when VCCIO3_PWR_DETECION='0' else c_vid_det; --, blink, no bank power, else VID control or rgpio
  
  -- if all pullup  old rev or I2C, else controlled by S4
  vid_det <= VID0 and VID1 and VID2;
	proc_vid_det: process (Clk, POR_DONE)
  begin 
    if POR_DONE = '0' then
      c_vid_det <= '0';
    elsif rising_edge(Clk) then
			c_vid_det <= n_vid_det;
		end if;
  end process;
  n_vid_det <= vid_det when PON='0' else c_vid_det;
  
	PGOOD <= 'Z';
  ----------------------------------------------------------
  -- JTAG
	-- JTAG Chain multiplexer, best is to bypass directly
    C_TMS 		<= M_TMS;
    C_TCK 		<= M_TCK;
    C_TDI 		<= M_TDI;
    M_TDO 		<=	C_TDO; 
	--M_TDO 		<= C_TDO when FMC_JTAGEN='1' else FMC_TDO;

    FMC_TMS 		<= '0'; -- M_TMS when FMC_JTAGEN='1' else '0';
    FMC_TCK 		<= '0'; -- M_TCK when FMC_JTAGEN='1' else '0';
    FMC_TDI 		<= '0'; -- M_TDI when FMC_JTAGEN='1' else '0';
	
    ADBUS4 			<= '0'; -- M_TCK; -- not needed!
	
  ----------------------------------------------------------
	--LED
	-- LED's 1..4 can always be used no matter jumper or power options
	-- LED's 5..8 can be on only if B2B large banks are powered
	--
	LED1 		<= vadj_led       when rgpio_active='0' else rgpio_out(1); 
  LED2 		<= MODE3          when rgpio_active='0' else rgpio_out(1);                      
	LED3 		<= not BDBUS0     when rgpio_active='0' else rgpio_out(2); -- UART Activity    
	LED4 		<= not MIO15      when rgpio_active='0' else rgpio_out(3); -- UART Activity                   
	LED5 		<= BOOTMODE       when rgpio_active='0' else rgpio_out(4); 
	LED6 		<= CM2            when rgpio_active='0' else rgpio_out(5);
	LED7 		<= X6             when rgpio_active='0' else rgpio_out(6); 
	LED8 		<= Y6             when rgpio_active='0' else rgpio_out(7); 



	-- Drive MAGJACK LEDS from FPGA I/O Pins
	PHY_LED1 <= Y2 when rgpio_active='0' else rgpio_out(8); -- keep NOSEQ as well for legacy reasons?
	PHY_LED2 <=	Y4 when rgpio_active='0' else rgpio_out(9);


----------------------------------------------------------
--
-- XO2 Internal oscillator
--
MyInternalOSC: OSCH
		-- synthesis translate_off
		GENERIC MAP ( NOM_FREQ => "24.18" )
		-- synthesis translate_on
		PORT MAP ( 
			STDBY		=> '0',
			OSC			=> osc_clk,
			SEDSTDBY	=> open
		);

	clk <= osc_clk;

----------------------------------------------------------
-- power on delay counter
Inst_PORESET: PORESET PORT MAP( 		
	SYSRST => ext_reset_n, 		
	RST => POR_DONE, 		
	CLK => Clk 	);

	process (POR_DONE, Clk)
	begin
		if POR_DONE = '0' then
			POR_DELAYED <= '0'; -- 
			por_cnt <= X"00000000";
		elsif Clk'event and Clk = '1' then
			por_cnt <= por_cnt + X"00000001";
			if por_cnt(23)='1' then
				POR_DELAYED <= '1';
			end if;
		end if;
	end process;

----------------------------------------------------------
-----------------
-- Remote GPIO
----------------
	--PMOD-GROUP
	rgpio_in(0) <= MIO10;
	rgpio_in(1) <= MIO11;
	rgpio_in(2) <= MIO12;
  rgpio_in(3) <= MIO13; 
	
	rgpio_in(4) <= MIO14; 
	rgpio_in(5) <= MIO15; 
	rgpio_in(6) <= PX6;
	rgpio_in(7) <= PX7;

  --Button-GROUP/--SWITCH-GROUP
	rgpio_in(8) <= S1; 
	rgpio_in(9) <= CM0;--S2; 
	rgpio_in(10) <= CM1; 
	rgpio_in(11) <= CM2; 
  --Others-GROUP
	rgpio_in(12) <= SD_WP; 
	rgpio_in(13) <= SD_DETECT; 
	rgpio_in(14) <= USB_OC; 
	rgpio_in(15) <= POK_FMC;
	
	rgpio_in(16) <= FMC_PRSNT;
	rgpio_in(17) <= PGOOD;
	rgpio_in(18) <= NOSEQ;
	rgpio_in(19) <= '0';
	
	rgpio_in(20) <= X6;
	rgpio_in(21) <= Y6;
	rgpio_in(22) <= '0';--HDMI_SCL;
	rgpio_in(23) <= '0';--HDMI_SDA;
	rgpio_in(24) <= '0';--reserved
	rgpio_in(25) <= '0';--reserved
	rgpio_in(26) <= '0';--reserved
	rgpio_in(27) <= '0';--reserved CPLD Firmware version as ASCI-sequence
  rgpio_in(31 downto 28) <= rgpio_active_code; -- reserved active detection
--RGPIO activation code
	rgpio_active <= '1' when (rgpio_out(31 downto 28)= rgpio_active_code) else '0';


-----------------------------------------------
--
-- RGPIO Pin Mapping
--
	rgpio_clk 	<= Y2;
	Y3 			    <= rgpio_txd;
	rgpio_rxd 	<= Y4;

MyRemoteGPIO: ddsrpi_slave
		PORT MAP ( 
			
        RX_CLK		=> rgpio_clk,
        TX_DATA 	=> rgpio_txd,
		RX_DATA 	=> rgpio_rxd,
        
        user_clk	=> open,  
        resetn 		=> '1',	
        ready 		=> open,
		--valid		=> open,
		
		gpio_in		=> rgpio_in,
		gpio_out	=> rgpio_out
		);

-----------------------------------------------
END ARCHITECTURE translated;